@@ -1,6 +1,7 @@
 #######################################################
 #	apc package
-#	Bent Nielsen, 8 Jan 2016, version 1.2
+#	Bent Nielsen,  7 Sep 2016, version 1.2.1.2
+#	Bent Nielsen,  8 Jan 2016, version 1.2
 #	Forecasting
 #######################################################
 #	Copyright 2016 Bent Nielsen
@@ -21,7 +22,9 @@
 #    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 #######################################################
 
-apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,covariance.output=FALSE,suppress.warning=TRUE)
+apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,quantiles=NULL,suppress.warning=TRUE)
+#	BN 6 Sep 2016	extention to od.poisson.response
+#	Based on Harnau and Nielsen (2016)
 #	BN 31 Jan 2016
 #	Based on Martinez Miranda, Nielsen and Nielsen (2015) JRSS-A
 #	Requires AC structure
@@ -35,8 +38,8 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 #										apc.fit$model.family=="poisson.response"
 #			sum.per.by.age
 #			sum.per.by.coh
-#			covariance.output			Logical. If TRUE gives output for computing the covariance matrix for
-#										cell-by-cell point forecasts. This matrix can be very large.
+#			quantiles					vector or NULL.
+#			suppress.warning			Logical.
 #	out
 #			linear.predictors.forecast	Vector. Linear predictors for forecast area
 #			index.trap.J				Matrix. age-coh coordinates for vector. Similar structure to index.trap in apc.index
@@ -57,10 +60,6 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 #			intercept.correction.per	Numeric.
 #										The intercept correction is constructed as the ratio of
 #										the sum of data entries for the last period and the sum of the corresponding fitted values
-#			covariance.proc				Matrix. Only if covariance.output=TRUE.
-#			covariance.est				Vector. Only if covariance.output=TRUE. 
-#										Note overall covariance matrix given by
-#										covariance.forecast=covariance.proc+diag(covariance.est)
 {	#	apc.forecast.ac
 	########################
 	#	get values
@@ -85,6 +84,8 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 	fitted.values			<- apc.fit$fitted.values
 	data.format				<- apc.fit$data.format
 	n.decimal				<- apc.fit$n.decimal
+	deviance                <- apc.fit$deviance
+	df.residual             <- apc.fit$df.residual	
 	#	derived values
 	per.forecast.J			<- age.max+coh.max-1-per.zero-per.max
 	n.data.J				<- (per.forecast.J*(per.forecast.J+1)) %/% 2	# note %/% has higher precedence than *
@@ -95,8 +96,8 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 	if(per.forecast.J==0)
 		return(cat("ERROR apc.forecast.ac: forecast area empty since per.forecast=age.max+coh.max-1-per.zero-per.max=0\n"))
 	distribution.forecasts	<- TRUE	
-	if(isTRUE(model.family=="poisson.response")==FALSE)
-	{	return(cat("WARNING apc.forecast.ac: model.family is not 'poisson.response': only point forecasts are generated\n"))
+	if(isTRUE(model.family %in% c("poisson.response","od.poisson.response"))==FALSE)
+	{	return(cat("WARNING apc.forecast.ac: model.family is not 'poisson.response' or 'od.poisson.response': only point forecasts are generated\n"))
 		distribution.forecasts	<- FALSE
 	}
 	if(is.null(sum.per.by.age)==FALSE && distribution.forecasts==TRUE)
@@ -139,7 +140,7 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 	#	Get design for forecast area
 	#	apc index list for forecast area
 	apc.index.J	<- list(	age.max		=  age.max			,		
-							per.max		=  age.max+coh.max-1,
+							per.max		=  age.max+coh.max-1-per.zero,
 							coh.max		=  coh.max			,
 							index.trap	=  index.trap.J		,
 							per.zero	=  per.zero			,
@@ -178,32 +179,67 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 		##############################
 		#	Get standard deviations
 		#	Use Martinez-Miranda, Nielsen, Nielsen (2005) JRSS-A, Appendix A.2, A.3
+		#	and Harnau, Nielsen (2016)
 		xi.dim	<- nrow(coefficients.canonical)
 		tau		<- sum(fitted.values)
 		#	MMNN (4.4) pi - multinomial probabilities
+		#	HN (5)
 		v.pi	<- fitted.values / tau
 		#	MMNN (A.2): H.2 insample design
-		design.2.average	<- colSums(diag(v.pi) %*% design[,2:xi.dim])
+		#	HN after (5)
+		design.2.average	<- colSums(v.pi * design[,2:xi.dim])
 		H.2		<- design[,2:xi.dim] - matrix(data=design.2.average,nrow=n.data,ncol=(xi.dim-1),byrow=T)
-		#	MMNN (A.3): information about xi_2 per obs 
-	#	i.1.xi.2	<- t(H.2) %*% diag(v.pi) %*% H.2
 		#	MMNN (A.12): H.2.J outofsample design
+		#	HN after (5)
 		v.pi.J		<- as.vector(exp(linear.predictors.J) / tau)
 		H.2.J		<- design.J[,2:xi.dim] - matrix(data=design.2.average,nrow=n.data.J,ncol=(xi.dim-1),byrow=T)
-	#	s2.est.J	<- H.2.J %*% solve(i.1.xi.2,t(H.2.J))
-	#	s2.est.J	<- tau * diag(v.pi.J) %*% s2.est.J %*% diag(v.pi.J)
-		s2.est.J	<- H.2.J %*% covariance.canonical[2:xi.dim,2:xi.dim] %*% t(H.2.J)
-		s2.est.J	<- (tau^2) * diag(v.pi.J) %*% s2.est.J %*% diag(v.pi.J)
-		s2.proc.J	<- tau * diag(v.pi.J)
-		s2.J		<- s2.proc.J + s2.est.J
+		m.pi.H.2.J	<- v.pi.J * H.2.J
+		#	note information per obs is covariance.canonical/tau 
+		s2.est.J	<- (tau^2) * m.pi.H.2.J %*%	covariance.canonical[2:xi.dim,2:xi.dim] %*% t(m.pi.H.2.J)		
 		##############################
 		#	Function to generate particular forecasts
+		#	6 Sep 2016 extended with od.poisson.response
 		function.sum.forecasts <- function(m.X,row.label=NULL)
-		{	forecast	<- m.X %*% response.forecast.cell[,1]
-			forecast	<- cbind(forecast,sqrt(diag(m.X %*% s2.J	  %*% t(m.X) )))
-			forecast	<- cbind(forecast,sqrt(diag(m.X %*% s2.proc.J %*% t(m.X) )))
-			forecast	<- cbind(forecast,sqrt(diag(m.X %*% s2.est.J  %*% t(m.X) )))
-			colnames(forecast)  <- c("forecast","se","se.proc","se.est")
+		#	in		m.X		matrix 	nrow=per.forecast.J,ncol=n.data.J
+		#	note	v.pi.J	vector	nrow=n.data.J
+		{	forecast		<- m.X %*% response.forecast.cell[,1]
+			#	HN sec 5.3: pi_A
+			v.pi.A	<- rowSums(m.X %*% v.pi.J)	#
+			forecast.est.J	<- diag(m.X %*% s2.est.J %*% t(m.X) )
+			forecast.proc.J	<- tau*v.pi.A
+			forecast.tau.J	<- tau*(v.pi.A^2)
+			forecast.J		<- forecast.proc.J + forecast.est.J
+			if(model.family == "od.poisson.response")
+			{	forecast.proc.J	<- forecast.proc.J*deviance/df.residual	
+				forecast.est.J	<- forecast.est.J *deviance/df.residual	
+				forecast.tau.J	<- forecast.tau.J *deviance/df.residual
+				forecast.J	<- forecast.proc.J + forecast.est.J + forecast.tau.J				
+			}
+			#	combine forecasts for poisson.response
+			forecast	<- cbind(forecast,sqrt(forecast.J))	
+			forecast	<- cbind(forecast,sqrt(forecast.proc.J))	
+			forecast	<- cbind(forecast,sqrt(forecast.est.J))	
+			col.names	<- c("forecast","se","se.proc","se.est")
+			#	combine forecasts adjusted when od.poisson.response
+			if(model.family == "od.poisson.response")
+			{	forecast	<- cbind(forecast,sqrt(forecast.tau.J))					
+				col.names		<- c(col.names,"tau.est")
+			}	
+			#	optional, add quantiles
+			if(is.null(quantiles)==FALSE)
+			{	if(model.family == "poisson.response")
+			  		for(row in 1:length(quantiles))
+					{	forecast	<- cbind(forecast,forecast[,1]+forecast[,2]*qnorm(quantiles[row]))
+						col.names	<- c(col.names,paste("n-",apc.internal.function.date.2.character(quantiles[row],3),sep=""))	
+					}	
+				if(model.family == "od.poisson.response")
+					for(row in 1:length(quantiles))
+					{	forecast	<- cbind(forecast,forecast[,1]+forecast[,2]*qt(quantiles[row],df.residual))
+						col.names	<- c(col.names,paste("t-",apc.internal.function.date.2.character(quantiles[row],3),sep=""))	
+					}
+			}
+			#	add labels
+			colnames(forecast)  <- col.names
 			if(is.null(row.label)==FALSE)
 				rownames(forecast)	<- row.label
 			return(forecast)	
@@ -336,46 +372,60 @@ apc.forecast.ac <- function(apc.fit,sum.per.by.age=NULL,sum.per.by.coh=NULL,cova
 	if(distribution.forecasts==FALSE)
 		return(list(linear.predictors.forecast		= linear.predictors.J	,
 					index.trap.J					= index.trap.J			))
-	if(distribution.forecasts==TRUE && covariance.output==FALSE)
+	if(distribution.forecasts==TRUE)
 		return(output.list)
-	if(distribution.forecasts==TRUE && covariance.output==TRUE)
-		return(c(output.list,
-						list(	covariance.proc				= s2.proc.J						,
-								covariance.est				= as.vector(exp(linear.predictors.J))
-								)))
 }	#	apc.forecast.ac
 
 #######################################################################################
 
 apc.forecast.ap <- function(apc.fit,extrapolation.type="I0",suppress.warning=TRUE)
+#	BN 2 May 2016
 #	BN 31 Jan 2016
 #	Computes point forecasts for a model with AP structure.
 #	This is done for the triangle which shares age and cohort indices with the data.
 #	in
 #			apc.fit						List. Output for apc.fit.model
 #										Note: apc.fit.model should be run first for a Poisson response model with AC structure so that
-#										apc.fit$model.design=="AC"
+#										apc.fit$model.design=="AP"
 #										apc.fit$model.family=="poisson.response"
 #			extrapolation.type			Character.
 #										For the extrapolation future Delta.beta are needed.
 #										"I0": Forecasts future beta by average of estimated beta (note, that future Delta.beta invariant to level)
 #										"I1": Forecasts future beta by most recent estimated beta
 #	out
-#			linear.predictors.forecast	Vector. Linear predictors for forecast area
+#			trap.predictors.forecast	Matrix. Includes estimates and point forecasts of linear predictor.
+#										That is design*coefficient. Same as the glm.fit value linear.predictors when there is no offset.
+#										Forecasts in lower right triangle. Trapezoid format.
 #			index.trap.J				Matrix. age-coh coordinates for vector. Similar structure to index.trap in apc.index
+#			D.xi.per.extrapolated 		Matrix. Extrapolated parameters.
+#										Dimension per.forecast.J=age.max+coh.max-1-per.zero-per.max rows, 1 column.
 #			trap.response.forecast		Matrix. Includes data and point forecasts. Forecasts in lower right triangle.
+#										Only implemented for model.family="binomial.dose.response"
+#										Equal to does in cell age,coh times 1 minus probability of surviving in that cell.
+#			trap.dose.forecast			Matrix. Includes data and point forecasts. Forecasts in lower right triangle.
+#										Dose in cell age,coh equal to dose in cell age-1,coh minus response in cell age-1,coh
+#										Only implemented for model.family="binomial.dose.response"
+#			trap.survival.forecast		Matrix. Point forecasts. Forecasts in lower right triangle
+#										Probability of surviving
+#										Only implemented for model.family="binomial.dose.response"
 {	#	apc.forecast.ap
 	########################
 	#	get values
 	age.max			        <- apc.fit$age.max			   	
 	per.max			        <- apc.fit$per.max	
-	coh.max			        <- apc.fit$coh.max	
+	coh.max			        <- apc.fit$coh.max
+	per1					<- apc.fit$per1
+	unit					<- apc.fit$unit
 	per.odd			        <- apc.fit$per.odd
 	per.zero		        <- apc.fit$per.zero
 	U				        <- apc.fit$U
 	index.trap		        <- apc.fit$index.trap
 	model.design			<- apc.fit$model.design
 	linear.predictors		<- apc.fit$linear.predictors
+	predictors				<- apc.fit$predictors
+	fitted.values			<- apc.fit$fitted.values
+	n.decimal				<- apc.fit$n.decimal
+	model.family			<- apc.fit$model.family
 	#	derived values
 	per.forecast.J			<- age.max+coh.max-1-per.zero-per.max
 	n.data.J				<- (per.forecast.J*(per.forecast.J+1)) %/% 2	# note %/% has higher precedence than *
@@ -396,14 +446,6 @@ apc.forecast.ap <- function(apc.fit,extrapolation.type="I0",suppress.warning=TRU
 	}
 	colnames(index.trap.J)	<- c("age","coh")
 	##############################
-	#	Generate forecast trapezoid
-	trap.linear.predictors	<- apc.data.list.subset(apc.fit,suppress.warning=suppress.warning)$response
-	trap.linear.predictors[index.trap]	<- linear.predictors
-	##############################
-	#	Extrapolate linear predictor for last diagonal to forecast area
-	for(i in 1:per.forecast.J)
-		trap.linear.predictors[age.max-per.forecast.J+i,coh.max-seq(0,i-1)] <- trap.linear.predictors[age.max-per.forecast.J+i,coh.max-i]
-	##############################
 	#	Get design for forecast area
 	#	apc index list for forecast area
 	apc.index.J	<- list(	age.max		=  age.max			,		
@@ -429,10 +471,13 @@ apc.forecast.ap <- function(apc.fit,extrapolation.type="I0",suppress.warning=TRU
 	#		thus Delta.beta_{per.max+1} = nu - beta_per.max , which is invariant to beta_1
 	#		thus Delpa.beta_{per.max+h} = 0 for h>1
 	D.xi.per.extrapolated		<- matrix(data=0,nrow=per.forecast.J,ncol=1)
+	#	Add labels	12 Apr 2016
+	rownames(D.xi.per.extrapolated)	<- paste("D_period",apc.internal.function.date.2.character(per1+(per.max-1+seq(1,per.forecast.J))*unit,n.decimal),sep="")
+	#   I0 forecast	
 	if(extrapolation.type=="I0")
 	{	D.xi.per	<- coefficients.dif[index.per.dif]
 		theta.per	<- cumsum(D.xi.per)
-		nu			<- sum(theta.per)/(1+length(D.xi.per))
+		nu			<- mean(c(0,theta.per))
 		D.xi.per.extrapolated[1]	<- nu - theta.per[length(theta.per)]
 	}
 	#	extrapolation.type == "I1"
@@ -443,19 +488,241 @@ apc.forecast.ap <- function(apc.fit,extrapolation.type="I0",suppress.warning=TRU
 	##############################
 	#	partial sum of forecast of Delta.beta
 	xi.per.extrapolated	<- cumsum(D.xi.per.extrapolated)
+	#	Add labels	16 Apr 2016
 	##############################
-	#	update linear predictor
+	#	GENERATE PREDICTORS
+	#	Generate forecast trapezoid: predictor
+	dim.names	<- dimnames(apc.data.list.subset(apc.fit,suppress.warning=suppress.warning)$response)
+	trap.predictors	<- matrix(nrow=age.max,ncol=coh.max,dimnames=dim.names)
+	#	Insert estimated predictors for data array
+	trap.predictors[index.trap]	<- predictors
+	#	Generate predictors for forecast array
 	for(i in 1:per.forecast.J)
+	{	#	First extrapolate predictor for last diagonal to forecast area
+		trap.predictors[age.max-per.forecast.J+i,coh.max-seq(0,i-1)] <- trap.predictors[age.max-per.forecast.J+i,coh.max-i]
+		#	Then add cumulative effect of forecast of Delta.beta		
 		for(j in 1:i)
-			trap.linear.predictors[age.max-i+j,coh.max-j+1] <- trap.linear.predictors[age.max-i+j,coh.max-j+1] + xi.per.extrapolated[per.forecast.J-i+1]
+			trap.predictors[age.max-i+j,coh.max-j+1] <- trap.predictors[age.max-i+j,coh.max-j+1] + xi.per.extrapolated[per.forecast.J-i+1]
+	}
 	##############################
-	return(list(	trap.linear.predictors.forecast	= trap.linear.predictors	,
-					index.trap.J				= index.trap.J				,
-					D.xi.per.extrapolated		= D.xi.per.extrapolated		))	
+	#	GENERATE FORECAST BY COMPONENT METHOD
+	#	FOR binomial.dose.response
+	trap.response		<- NULL
+	trap.dose			<- NULL
+	trap.survival		<- NULL
+	if(model.family=="binomial.dose.response")
+	{	#	Generate forecast trapezoid
+		trap.response	<- trap.dose	<- trap.survival	<- matrix(nrow=age.max,ncol=coh.max,dimnames=dim.names)
+		#	Insert values for data array, but recast in trapezoid format
+		data.list.subset	<- apc.data.list.subset(apc.fit,suppress.warning=suppress.warning)
+		trap.response		<- data.list.subset$response
+		trap.dose			<- data.list.subset$dose
+		#	Generate survival probabilities, dose, respone
+		for(i in 1:per.forecast.J)
+		{	trap.survival[age.max-per.forecast.J+i,coh.max-seq(0,i-1)] 	<- 1/(1+exp(trap.predictors[age.max-per.forecast.J+i,coh.max-seq(0,i-1)]))
+			trap.dose[age.max-per.forecast.J+i,coh.max-seq(0,i-1)]		<- trap.dose[age.max-per.forecast.J+i-1,coh.max-seq(0,i-1)]	- trap.response[age.max-per.forecast.J+i-1,coh.max-seq(0,i-1)]
+			trap.response[age.max-per.forecast.J+i,coh.max-seq(0,i-1)]	<- trap.dose[age.max-per.forecast.J+i,coh.max-seq(0,i-1)] * (1-trap.survival[age.max-per.forecast.J+i,coh.max-seq(0,i-1)])
+		}
+	}
+	##############################
+	return(list(trap.predictors.forecast	= trap.predictors			,
+				index.trap.J				= index.trap.J				,
+				D.xi.per.extrapolated		= D.xi.per.extrapolated		,
+				trap.response.forecast		= trap.response				,
+				trap.dose.forecast			= trap.dose					,
+				trap.survival.forecast		= trap.survival				))	
 }	#	apc.forecast.ap
 
 #######################################################################################
 
+apc.forecast.apc <- function(apc.fit,extrapolation.type="I0",suppress.warning=TRUE)
+#	BN 10 Sep 2016
+#	Computes point forecasts for a model with AP structure
+#	This is done for the triangle which shares age and cohort indices with the data.
+#	in
+#			apc.fit						List. Output for apc.fit.model
+#										Note: apc.fit.model should be run first for a Poisson response model with AC structure so that
+#										apc.fit$model.design=="APC"
+#										apc.fit$model.family=="poisson.response"
+#			extrapolation.type			Character.
+#										For the extrapolation future Delta.beta are needed.
+#										"I2": Forecasts future DDbeta by 0
+#										"I1": Forecasts future DDbeta as follows.
+#												Compute Dbeta=cumsum(DDbeta) for j=3,...,J.
+#												This determines Dbeta upto arbitrary level
+#												Compute average mean(Dbeta)
+#												Forecast DDbeta[J+1]=mean(Dbeta)-Dbeta[J]
+#												Forecast DDbeta[J+h]=0 for h>1
+#												This forecast is invariant to arbitrary level
+#										"I0": Forecasts future DDbeta as follows.
+#												Compute beta=cumsum(cumsum(DDbeta)) for j=3,...,J.
+#												This determines beta upto arbitrary linear trend
+#												Regress on 1 and demeaned trend=j-(n+1)/2
+#												giving estimates mu1 and mu2
+#												Forecast beta[J+1]=mu1 + mu2*(n+1-(n+1)/2)
+#												Forecast beta[J+2]=mu1 + mu2*(n+2-(n+1)/2)
+#												Forecast DDbeta[J+h]=beta[J+h]-2*beta[J+h-1]+beta[J+h-2] for h=1,2
+#												Forecast DDbeta[J+h]=0 for h>2
+#												This forecast is invariant to arbitrary linear trend
+#	out
+{	#	apc.forecast.apc
+	########################
+	#	get values
+	age.max			        <- apc.fit$age.max			   	
+	per.max			        <- apc.fit$per.max	
+	coh.max			        <- apc.fit$coh.max
+	age1					<- apc.fit$age1
+	per1					<- apc.fit$per1
+	coh1					<- apc.fit$coh1
+	unit					<- apc.fit$unit
+	per.odd			        <- apc.fit$per.odd
+	per.zero		        <- apc.fit$per.zero
+	U				        <- apc.fit$U
+	index.trap		        <- apc.fit$index.trap
+	model.design			<- apc.fit$model.design
+#	linear.predictors		<- apc.fit$linear.predictors
+	predictors				<- apc.fit$predictors
+#	fitted.values			<- apc.fit$fitted.values
+	n.decimal				<- apc.fit$n.decimal
+	model.family			<- apc.fit$model.family
+	coefficients.canonical	<- apc.fit$coefficients.canonical
+	index.age				<- apc.fit$index.age
+	index.per				<- apc.fit$index.per
+	index.coh				<- apc.fit$index.coh
+#	#	derived values
+	per.forecast.J			<- age.max+coh.max-1-per.zero-per.max
+	n.data.J				<- (per.forecast.J*(per.forecast.J+1)) %/% 2	# note %/% has higher precedence than *
+	##############################
+	#	Check Input
+	if(isTRUE(model.design=="APC")==FALSE)
+		return(cat("ERROR apc.forecast.apc: model.design must be 'APC'\n"))
+	if(per.forecast.J==0)
+		return(cat("ERROR apc.forecast.apc: forecast area empty since per.forecast=age.max+coh.max-1-per.zero-per.max=0\n"))
+	##############################
+	#	index for forecast area
+	index.trap.J		<- matrix(nrow=n.data.J,ncol=2)
+	s	<- 0
+	for(row in 1:per.forecast.J)
+	{	index.trap.J[(s+1):(s+row),1]	<- seq(age.max-row+1,age.max)	# age
+		index.trap.J[(s+1):(s+row),2]	<- coh.max-per.forecast.J+row	# cohort
+		s	<- s + row
+	}
+	colnames(index.trap.J)	<- c("age","coh")
+	##############################
+	#	Get design for forecast area
+	#	apc index list for forecast area
+	apc.index.J	<- list(	age.max		=  age.max			,		
+							per.max		=  age.max+coh.max-1-per.zero,
+							coh.max		=  coh.max			,
+							index.trap	=  index.trap.J		,
+							per.zero	=  per.zero			,
+							per.odd		=  per.odd			,
+							U			=  U				,
+							n.data		=  n.data.J			)
+	#	get design matrix
+   	design.J	<- apc.get.design(apc.index.J,model.design)$design
+	##############################
+	#	Extrapolate period parameters
+	#	get estimates
+	xi.per.dd	<- coefficients.canonical[index.per,1]
+	#	define extrapolation array
+	xi.per.dd.extrapolated <- matrix(data=0,nrow=per.forecast.J,ncol=1)
+	#	add row names
+	rownames(xi.per.dd.extrapolated)	<- paste("DD_period_",apc.internal.function.date.2.character(per1+(per.max-1+seq(1,per.forecast.J))*unit,n.decimal),"_x",sep="")
+	#	extrapolation.type == "I2"
+	#	nothing happens
+	#	extrapolation.type == "I1"
+	if(extrapolation.type == "I1")	
+		xi.per.dd.extrapolated[1,1] <-  mean(cumsum(xi.per.dd)) - cumsum(xi.per.dd)[length(xi.per.dd)] 
+	#	extrapolation.type == "I0"
+	if(extrapolation.type == "I0")	
+	{	n	<- length(xi.per.dd)
+		y	<- cumsum(cumsum(xi.per.dd))
+		trend	<- seq(1,n)-(n+1)/2
+		mu1 <- mean(y)
+		mu2 <- (trend %*% y) / (trend %*% trend)
+		y_n_plus_1	<- mu1 + mu2 * (n+1)/2
+		y_n_plus_2	<- mu1 + mu2 * (n+3)/2
+		xi.per.dd.extrapolated[1,1]	<- y_n_plus_1 - 2*y[n] + y[n-1]
+		xi.per.dd.extrapolated[2,1]	<- y_n_plus_2 - 2*y_n_plus_1 + y[n]	
+	}	
+	################################
+	#	Extrapolate xi parameters
+	#	merge extrapolated DDbeta with estimated xi
+	xi.extrapolated	<- c(coefficients.canonical[c(1:3,index.age,index.per),1],xi.per.dd.extrapolated[,1],coefficients.canonical[c(index.coh),1])
+	xi.extrapolated <- as.matrix(xi.extrapolated)
+	##############################
+	#	GENERATE PREDICTORS
+	#	Generate forecast trapezoid: predictor
+	dim.names	<- dimnames(apc.data.list.subset(apc.fit,suppress.warning=suppress.warning)$response)
+	trap.linear.predictors	<- matrix(nrow=age.max,ncol=coh.max,dimnames=dim.names)
+	#	Insert estimated predictors for data array
+	trap.linear.predictors[index.trap]	<- predictors
+	#	Generate predictors for forecast array
+	linear.predictors.J						<- design.J %*% xi.extrapolated[,]
+	trap.linear.predictors[index.trap.J]	<- design.J %*% xi.extrapolated[,]
+	#	Go to original scale
+	if(model.family %in% c("poisson.response","od.poisson.response"))
+		trap.response.forecast	<- exp(trap.linear.predictors)
+	else
+		trap.response.forecast	<- NULL
+	##############################
+	#	Generate partial sum forecasts
+	#	matrices
+	if(model.family %in% c("poisson.response","od.poisson.response"))
+	{	m.partial.sum.age	<- matrix(data=0,nrow=per.forecast.J,ncol=n.data.J)
+		m.partial.sum.per	<- matrix(data=0,nrow=per.forecast.J,ncol=n.data.J)
+		m.partial.sum.coh	<- matrix(data=0,nrow=per.forecast.J,ncol=n.data.J)
+		m.partial.sum.all	<- matrix(data=1,nrow=1,      		 ncol=n.data.J)
+		s	<- 0
+		for(row in 1:per.forecast.J)
+		{	for(age in 1:row)
+				m.partial.sum.age[per.forecast.J-row+age,s+age]	<- 1
+			for(per in 1:row)
+				m.partial.sum.per[per,s+per]		<- 1
+			m.partial.sum.coh[row,(s+1):(s+row)]	<- 1	
+			s	<- s + row
+		}
+		#	forecasts
+		response.forecast.age	<- m.partial.sum.age %*% exp( design.J %*% xi.extrapolated[,] )
+		response.forecast.per	<- m.partial.sum.per %*% exp( design.J %*% xi.extrapolated[,] )
+		response.forecast.coh	<- m.partial.sum.coh %*% exp( design.J %*% xi.extrapolated[,] )
+		response.forecast.all	<- m.partial.sum.all %*% exp( design.J %*% xi.extrapolated[,] )
+		#	labels
+		label.age	<- c(paste("age_",apc.internal.function.date.2.character(age1+seq(age.max-per.forecast.J,age.max-1)*unit,n.decimal),sep=""))
+		label.per	<- c(paste("per_",apc.internal.function.date.2.character(per1+seq(per.max,per.max+per.forecast.J-1)*unit,n.decimal),sep=""))
+		label.coh	<- c(paste("coh_",apc.internal.function.date.2.character(coh1+seq(coh.max-per.forecast.J,coh.max-1)*unit,n.decimal),sep=""))
+		label.all	<-  "all"
+		rownames(response.forecast.age)	<- label.age
+		rownames(response.forecast.per)	<- label.per
+		rownames(response.forecast.coh)	<- label.coh
+		rownames(response.forecast.all)	<- label.all
+		colnames(response.forecast.age)	<- "forecast"
+		colnames(response.forecast.per)	<- "forecast"
+		colnames(response.forecast.coh)	<- "forecast"
+		colnames(response.forecast.all)	<- "forecast"
+	}
+	else
+	{	response.forecast.age	<- NULL
+		response.forecast.per	<- NULL
+		response.forecast.coh	<- NULL
+		response.forecast.all	<- NULL
+	}
+	################################
+	return(list(linear.predictors.forecast		= linear.predictors.J		,
+				index.trap.J					= index.trap.J				,
+				trap.linear.predictors.forecast	= trap.linear.predictors	,
+				trap.response.forecast			= trap.response.forecast	,
+				response.forecast.age			= response.forecast.age		,
+				response.forecast.per			= response.forecast.per		,
+				response.forecast.coh			= response.forecast.coh		,
+				response.forecast.all			= response.forecast.all		,
+				xi.per.dd.extrapolated			= xi.per.dd.extrapolated	,
+				xi.extrapolated					= xi.extrapolated			))
+}	#	apc.forecast.apc
+
+#######################################################################################
+
 
 apc.polygon <- function(m.forecast,x.origin=1,
 	plot.se=TRUE,plot.se.proc=FALSE,plot.se.est=FALSE,
